/*
* Copyright (C) 2005, Pasi P. Liimatainen
* Distributable under GNU General Public License (v2)
* See terms of license at http://www.gnu.org
*/

function GetTagsRequester(aParent) {
  // Executed when the URL has been got.
  this.parent = aParent;
  this.asyncGetURLResponse = function(resp) {
    if (resp != null) {
      this.parent.mGettingTagsFailed = false;
      this.parent.tags = {};
      this.parent.tagNames = [];
      var root = resp.documentElement;
      var myTags = root.getElementsByTagName("tag");
      for (var i = 0; i < myTags.length; i++) {
        var tagName = myTags[i].getAttribute("tag");
        var tagCount = myTags[i].getAttribute("count");
        this.parent.tags[tagName] = parseInt(tagCount);
        this.parent.tagNames.push(tagName);
      }
    }
    this.callback(this.callbackParam);
  };
  // Process error.
  this.asyncGetURLError = function(error) {
    this.parent.mGettingTagsFailed = true;
    return;
  };
};

function Tags(aDeliciousService) {
  this.tags = {};
  this.tagNames = [];
  this.deliciousService = aDeliciousService;
  this.mGettingTagsFailed = false;
  this.__defineGetter__("gettingTagsFailed", function() {return this.mGettingTagsFailed;});

  this.getTagsRequester = new GetTagsRequester(this);

  this.preloadTags = function(aCallback, aParam, aUsername, aPassword) {
    this.getTagsRequester.callback = aCallback; //TODO A nicer way to set callback and callbackParam.
    this.getTagsRequester.callbackParam = aParam;
    if (this.tagNames.length == 0) {
      this.deliciousService.getTagsForUser(this.getTagsRequester, aUsername, aPassword);
    } else {
      if (aCallback != null) {
        aCallback(aParam);
      }
    }
  };

  this.getTags = function() {
    if (this.tagNames.length == 0) {
      if (this.gettingTagsFailed) {
        throw("del.icio.us has failed to get tags from the server.");
      }
      const xmlDOM = this.deliciousService.getTags();
      var root = xmlDOM.documentElement;
      var myTags = root.getElementsByTagName("tag");
      this.tags = {};
      this.tagNames = [];
      for (var i = 0; i < myTags.length; i++) {
        var tagName = myTags[i].getAttribute("tag");
        var tagCount = myTags[i].getAttribute("count");
        this.tags[tagName] = parseInt(tagCount);
        this.tagNames.push(tagName);
      }
    }
    return this.tags;
  };

  this.getTagNames = function() {
    if (this.tagNames.length == 0) {
      try {
        this.getTags();
      } catch(e) {}
    }
    return this.tagNames;
  };

  this.getTagNamesSorted = function() {
    if (this.tagNames.length == 0) {
      try {
        this.getTags();
      } catch(e) {}
    }
    var lower = [];
    var map = {};
    for (t in this.tags) {
      map[t.toLowerCase()] = t;
      lower.push(t.toLowerCase());
    }
    lower = lower.sort();
    var sorted = [];
    for (t in lower) {
      sorted.push(map[lower[t]]);
    }
    return sorted;
  };

  this.getTagNamesSortedByFrequency = function() {
    if (this.tagNames.length == 0) {
      try {
        this.getTags();
      } catch(e) {}
    }
    var map = {};
    var frequencies = 0;
    for (t in this.tags) {
      var count = this.tags[t];
      if (map[count]) {
        map[count].push(t);
      } else {
        map[count] = [t];
        frequencies++;
      }
    }
    var tmpmap = [];
    for (e in map) {
      tmpmap.push([e, map[e]]);
    }
    map = tmpmap.sort(function(a,b){return parseInt(b)-parseInt(a);});
    var sorted = [];
    for (m in map) {
      sorted = sorted.concat(map[m][1]);
    }
    return sorted;
  };

  this.addTags = function(aTags) {
    var usersTags = this.getTags();
    for (t in aTags) {
      if (usersTags[aTags[t]] == null) {
        usersTags[aTags[t]] = 1;
        this.tagNames.push(aTags[t]);
      } else {
        usersTags[aTags[t]] += 1;
      }
    }
  };

  this.asXml = function(aSorted) {
    if (this.tagNames.length == 0) {
      try {
        this.getTags();
      } catch(e) {}
    }
    var tagXML = '<tags>';
    var tags = this.tags;
    var tagNames = this.tagNames;
    var lower = [];
    var map = {};
    for (t in tags) {
      map[t.toLowerCase()] = t;
      lower.push(t.toLowerCase());
    }
    tagNames = aSorted ? lower.sort() : lower;
    for (t in tagNames) {
      tagXML += '<tag count="' + tags[map[tagNames[t]]] + '" tag="'+map[tagNames[t]]+'"/>';
    }
    tagXML += '</tags>';
    return tagXML;
  };

  this.reload = function(aCallback, aUsername, aPassword) {
    this.tags = {};
    this.tagNames = [];
    this.preloadTags(aCallback, 0, aUsername, aPassword);
  }
}
